// python wrapper for vtkFloatingPointExceptions
//
#define VTK_WRAPPING_CXX
#define VTK_STREAMS_FWD_ONLY
#include "vtkPythonArgs.h"
#include "vtkPythonOverload.h"
#include "vtkConfigure.h"
#include <cstddef>
#include <sstream>
#include "vtkVariant.h"
#include "vtkFloatingPointExceptions.h"

extern "C" { VTK_ABI_EXPORT void PyVTKAddFile_vtkFloatingPointExceptions(PyObject *); }

static const char **PyvtkFloatingPointExceptions_Doc();


static PyObject *
PyvtkFloatingPointExceptions_Enable(PyObject *, PyObject *args)
{
  vtkPythonArgs ap(args, "Enable");

  PyObject *result = NULL;

  if (ap.CheckArgCount(0))
  {
    vtkFloatingPointExceptions::Enable();

    if (!ap.ErrorOccurred())
    {
      result = ap.BuildNone();
    }
  }

  return result;
}


static PyObject *
PyvtkFloatingPointExceptions_Disable(PyObject *, PyObject *args)
{
  vtkPythonArgs ap(args, "Disable");

  PyObject *result = NULL;

  if (ap.CheckArgCount(0))
  {
    vtkFloatingPointExceptions::Disable();

    if (!ap.ErrorOccurred())
    {
      result = ap.BuildNone();
    }
  }

  return result;
}

static PyMethodDef PyvtkFloatingPointExceptions_Methods[] = {
  {"Enable", PyvtkFloatingPointExceptions_Enable, METH_VARARGS,
   "V.Enable()\nC++: static void Enable()\n\nEnable floating point exceptions.\n"},
  {"Disable", PyvtkFloatingPointExceptions_Disable, METH_VARARGS,
   "V.Disable()\nC++: static void Disable()\n\nDisable floating point exceptions.\n"},
  {NULL, NULL, 0, NULL}
};

static PyObject *
PyvtkFloatingPointExceptions_New(PyTypeObject *, PyObject *, PyObject *)
{
  PyErr_SetString(PyExc_TypeError,
                  "this class cannot be instantiated");

  return NULL;
}

static void PyvtkFloatingPointExceptions_Delete(PyObject *self)
{
  PyVTKSpecialObject *obj = (PyVTKSpecialObject *)self;
  delete static_cast<vtkFloatingPointExceptions *>(obj->vtk_ptr);
  PyObject_Del(self);
}

static Py_hash_t PyvtkFloatingPointExceptions_Hash(PyObject *self)
{
#if PY_VERSION_HEX >= 0x020600B2
  return PyObject_HashNotImplemented(self);
#else
  char text[256];
  sprintf(text, "unhashable type: '%s'", Py_TYPE(self)->tp_name);
  PyErr_SetString(PyExc_TypeError, text);
  return -1;
#endif
}

static PyTypeObject PyvtkFloatingPointExceptions_Type = {
  PyVarObject_HEAD_INIT(&PyType_Type, 0)
  "vtkCommonCorePython.vtkFloatingPointExceptions", // tp_name
  sizeof(PyVTKSpecialObject), // tp_basicsize
  0, // tp_itemsize
  PyvtkFloatingPointExceptions_Delete, // tp_dealloc
  0, // tp_print
  0, // tp_getattr
  0, // tp_setattr
  0, // tp_compare
  PyVTKSpecialObject_Repr, // tp_repr
  0, // tp_as_number
  0, // tp_as_sequence
  0, // tp_as_mapping
  PyvtkFloatingPointExceptions_Hash, // tp_hash
  0, // tp_call
  0, // tp_str
  PyObject_GenericGetAttr, // tp_getattro
  0, // tp_setattro
  0, // tp_as_buffer
  Py_TPFLAGS_DEFAULT, // tp_flags
  0, // tp_doc
  0, // tp_traverse
  0, // tp_clear
  0, // tp_richcompare
  0, // tp_weaklistoffset
  0, // tp_iter
  0, // tp_iternext
  0, // tp_methods
  0, // tp_members
  0, // tp_getset
  0, // tp_base
  0, // tp_dict
  0, // tp_descr_get
  0, // tp_descr_set
  0, // tp_dictoffset
  0, // tp_init
  0, // tp_alloc
  PyvtkFloatingPointExceptions_New, // tp_new
  PyObject_Del, // tp_free
  0, // tp_is_gc
  0, // tp_bases
  0, // tp_mro
  0, // tp_cache
  0, // tp_subclasses
  0, // tp_weaklist
  VTK_WRAP_PYTHON_SUPPRESS_UNINITIALIZED
};

extern "C" { VTK_ABI_EXPORT PyObject *PyvtkFloatingPointExceptions_TypeNew(); }

PyObject *PyvtkFloatingPointExceptions_TypeNew()
{
  PyVTKSpecialType_Add(
    &PyvtkFloatingPointExceptions_Type,
    PyvtkFloatingPointExceptions_Methods,
    NULL,
    PyvtkFloatingPointExceptions_Doc(), NULL);

  PyTypeObject *pytype = &PyvtkFloatingPointExceptions_Type;

  if ((pytype->tp_flags & Py_TPFLAGS_READY) != 0)
  {
    return (PyObject *)pytype;
  }

  PyType_Ready(pytype);
  return (PyObject *)pytype;
}

const char **PyvtkFloatingPointExceptions_Doc()
{
  static const char *docstring[] = {
    "vtkFloatingPointExceptions - Deal with floating-point exceptions\n\n",
    "Right now it is really basic and it only provides a function to\nenable floating point exceptions on some compilers. Note that Borland\nC++ has floating-point exceptions by default, not Visual studio nor\ngcc. It is mainly use to optionally enable floating point exceptions\nin the C++ tests.\n\n",
    NULL
  };

  return docstring;
}

void PyVTKAddFile_vtkFloatingPointExceptions(
  PyObject *dict)
{
  PyObject *o;
  o = PyvtkFloatingPointExceptions_TypeNew();

  if (o && PyDict_SetItemString(dict, "vtkFloatingPointExceptions", o) != 0)
  {
    Py_DECREF(o);
  }

}

